︠8b38344f-19fe-4546-8969-ccb34ed2e93ai︠
%html
<h1><center>Elliptic Curve Embedding</center></h1>
︡19c99d9c-2aea-4cd9-836a-a37410d2cf63︡{"done":true,"html":"<h1><center>Elliptic Curve Embedding</center></h1>"}
︠2977521a-5da3-4aa9-82a0-35e5d5002716︠
import math
import numpy
import itertools

############################################################################################################################
# For all these procedures we are working with elliptic curve (EC) groups over Z_p where p is a prime number larger than 3.#
# An Elliptic Curve group will be represented by a triple [A,B,p] where                                                    #
#      (1) p is a prime number larger than 3                                                                               #
#      (2) A is the coefficient of x and                                                                                   #
#      (3) B is the constant term in  y^2 = x^3 + A*x + B mod p.                                                           #
#                                                                                                                          #
# A point in the group will be represented as a pair [x,y] where                                                           #
#      (1)  x is the x-coordinate of a solution                                                                            #
#      (2)  y is the y-coordinate of a solution                                                                            #
#      (3)  The identity will be represented by []                                                                         #
############################################################################################################################


︡566a5a80-cdad-4d14-94bc-615be1942064︡
︠4bd60202-1686-4c25-aadd-4595b24fad3ci︠
%html
<ol>
<li><strong>Koblitz Embedding of a message into an Elliptic Curve Group:</strong> ECEmbed(Message, Group, Tollerance Parameter)</li>
<li><strong>Koblitz Unembedding of an Eliptic Curve group element to a message:</strong> ECUnembed(Group Element, Tollerance Parameter) </li>
</ol>
︡50c5db92-1776-4279-900d-ebcf8bd25e49︡{"done":true,"html":"<ol>\n<li><strong>Koblitz Embedding of a message into an Elliptic Curve Group:</strong> ECEmbed(Message, Group, Tollerance Parameter)</li>\n<li><strong>Koblitz Unembedding of an Eliptic Curve group element to a message:</strong> ECUnembed(Group Element, Tollerance Parameter) </li>\n</ol>"}
︠499b59aa-8826-4660-9f0b-6b88e932f2f7︠


def ASCIIPad(Mess,Mod):
    K = []
    for letter in Mess:
        K.append(ord(letter))
    L = Mod.ndigits()
    l = len(K)
    y = ZZ(math.floor(L/3))
    count = 0
    padded = []
    buffer = ""
    for numChar in K:
        numChar+=100
        buffer+=str(numChar)
        count+=1
        if count==y:
            padded.append(ZZ(buffer))
            buffer = ""
            count = 0
    if len(buffer)>0:
        padded.append(ZZ(buffer))
    return padded

def ASCIIDepad(Number):
    N = "";
    n = Number.ndigits() % 3;
    if (n > 0):
        print("This is not a padded ASCII string\n");
    else:
        L = [((Number - (Number % (1000^i)))/1000^i)%1000 - 100 for i in range(Number.ndigits()/3)];
        for i in range(Number.ndigits()/3):
            N = chr(L[i]) + N;
    return(N);

def ECEmbed (Message, gp, tolerance):
    p = ZZ(math.floor(gp[2] / (tolerance + 1)))
    M = ASCIIPad(Message, p)
    packets = len(M)
    pointlist = [0]*packets
    for j in range(0, packets ):
        N = M[j]
        pointlist[j] = ECSearch(tolerance * N, tolerance * (N + 1) - 1, gp)
    return(pointlist)


def ECUnembed (pointlist, tolerance):
    k = len(pointlist)
    paddedasciilist=[0]*k
    for j in range(0, k ):
        pointlist[j][0]=ZZ(pointlist[j][0])
        toType = ZZ(QQ((pointlist[j][0])/tolerance).floor())
        paddedasciilist[j] = ((pointlist[j][0])/tolerance).floor()
    returnStr = ""
    for paddedItem in paddedasciilist:
        buffer = ASCIIDepad(paddedItem)
        returnStr+= buffer
    return returnStr
︡3b4cb93e-4d3c-4ea0-ae47-2f143fce4ab7︡{"done":true}
︠d6a5358e-7302-4a22-916c-b020e46bb3aei︠
%html
<strong>Example.</strong> Embed the plaintext "Data Privacy Day is an international event that occurs every year on 28 January" into the group $$y^2=x^3+2x+12\; mod\;111372849328749456464564594837594837598435984375503$$ using a tolerance parameter T=40. The embeded plaintext is represented by 5 points in the elliptic curve group:

<ul>
    <li> [6727888647885287208568208727887968845286727888843, 32854152900797780223685776356973672727544281140438]</li> <li> [5288208605287888405288208408648048568407888648200, 27814776229677670024234415355169389176662105417080]</li> <li> [8448407888325288048728048408645288648167888645282, 108722806482736255145408973542651102334000896330604]</li> <li> [8447967968688568605288048728048568845288848047881, 46525767831756816282648267526217685495666590869802]</li> <li> 
    [8565288448405286006245286967888408687888568843, 95647711591715781059539655025908232090100039972726]</li> 

</ul>

︡6e4633ea-5f2f-4aba-a2c0-f3f1340f5add︡{"done":true,"html":"<strong>Example.</strong> Embed the plaintext \"Data Privacy Day is an international event that occurs every year on 28 January\" into the group $$y^2=x^3+2x+12\\; mod\\;111372849328749456464564594837594837598435984375503$$ using a tolerance parameter T=40. The embeded plaintext is represented by 5 points in the elliptic curve group:\n\n<ul>\n    <li> [6727888647885287208568208727887968845286727888843, 32854152900797780223685776356973672727544281140438]</li> <li> [5288208605287888405288208408648048568407888648200, 27814776229677670024234415355169389176662105417080]</li> <li> [8448407888325288048728048408645288648167888645282, 108722806482736255145408973542651102334000896330604]</li> <li> [8447967968688568605288048728048568845288848047881, 46525767831756816282648267526217685495666590869802]</li> <li> \n    [8565288448405286006245286967888408687888568843, 95647711591715781059539655025908232090100039972726]</li> \n\n</ul>"}
︠9271181e-050b-42f8-916e-ca3ff6b4e104s︠

TextPoint=ECEmbed("Data Privacy Day is an international event that occurs every year on 28 January",[2,12,next_prime(111372849328749456464564594837594837598435984375311)],40)
print("The embedded message is a string of 5 elliptic curve points\n", TextPoint)

︡230189ed-5020-445f-83e4-cbe91771e074︡{"stdout":"The embedded message is a string of 5 elliptic curve points\n [[6727888647885287208568208727887968845286727888843, 32854152900797780223685776356973672727544281140438], [5288208605287888405288208408648048568407888648200, 27814776229677670024234415355169389176662105417080], [8448407888325288048728048408645288648167888645282, 108722806482736255145408973542651102334000896330604], [8447967968688568605288048728048568845288848047881, 46525767831756816282648267526217685495666590869802], [8565288448405286006245286967888408687888568843, 95647711591715781059539655025908232090100039972726]]\n"}︡{"done":true}
︠08f3d021-0f2a-4970-bca8-22893d78ba06︠
##################################################################################################################################################
#The ECUnembed function converts a point on the elliptic curve to a readable message. The tollerance parameter used in ECUnembed must the same as# 
# the tollerance parameter used in ECEmbed. The tolerance parameter T is an interger between 40 < T < 50.                                        #
##################################################################################################################################################

ECUnembed (TextPoint,40)
︡d9d4f17c-4bc3-459b-aaf4-9e0d1ef7e67d︡{"stdout":"'Data Privacy Day is an international event that occurs every year on 28 January'\n"}︡{"done":true}
︠19948ebf-badf-4bc4-84ca-9b663e265d3ds︠
TextPoint[0]
ECUnembed ([TextPoint[0]],40)
︡c3896e1c-07d8-4188-8666-8bab49d50b32︡{"stdout":"[6727888647885287208568208727887968845286727888843, 32854152900797780223685776356973672727544281140438]\n"}︡{"stdout":"'Data Privacy Day'\n"}︡{"done":true}
︠c99e8cbf-cf1b-4220-a769-089488e9bb8es︠
TextPoint[1]
ECUnembed ([TextPoint[1]],40)
︡8195d5f1-fda0-4444-9ff1-aa80b9a268c1︡{"stdout":"[5288208605287888405288208408648048568407888648200, 27814776229677670024234415355169389176662105417080]\n"}︡{"stdout":"' is an internati'\n"}︡{"done":true}
︠411fde1c-ef29-49ed-85ad-f3cb2b43ff9bs︠
TextPoint[2]
ECUnembed ([TextPoint[2]],40)
︡b8dbc4d6-4946-477a-bae0-fe4bf4e195db︡{"stdout":"[8448407888325288048728048408645288648167888645282, 108722806482736255145408973542651102334000896330604]\n"}︡{"stdout":"'onal event that '\n"}︡{"done":true}
︠d31b7512-3ebd-49ff-87e9-f6af4e190966s︠
TextPoint[3]
ECUnembed ([TextPoint[3]],40)
︡92b7458d-802d-4bb7-bc8f-440a264119f0︡{"stdout":"[8447967968688568605288048728048568845288848047881, 46525767831756816282648267526217685495666590869802]\n"}︡{"stdout":"'occurs every yea'\n"}︡{"done":true}
︠d43b0a4a-e64f-4582-b9a6-d261d09c217as︠
TextPoint[4]
ECUnembed ([TextPoint[4]],40)
︡f5fddd5a-161f-4743-8e30-5639da3b028f︡{"stdout":"[8565288448405286006245286967888408687888568843, 95647711591715781059539655025908232090100039972726]\n"}︡{"stdout":"'r on 28 January'\n"}︡{"done":true}









